@@ -18,6 +18,7 @@
 #include <sys/mman.h>
 #include <stdarg.h>
 #include <dirent.h>
+#include <sys/utsname.h>
 
 #define MAX_FILES 1024
 #define PIDFILE "/var/run/memlockd.pid"
@@ -101,9 +102,38 @@ int open_map(int fd, struct stat *sb, const char * const name)
   return 1;
 }
 
-// return 0 for no file mapped and 1 for file mapped
-int open_file(const char * const name, int no_error_non_exist)
+const char *archname = NULL;
+
+// returns NULL if old name is OK. newname must be freed by caller
+char *make_new_name(const char *orig_name)
 {
+  const char *tmp = strstr(orig_name, "$ARCH");
+  if(!tmp)
+    return NULL;
+
+  int offset = tmp - orig_name;
+  char *newname = (char *)malloc(strlen(orig_name) + 1 + strlen(archname));
+  if(!newname)
+  {
+    log(LOG_ERR, "malloc() failure");
+    exit(1);
+  }
+  memcpy(newname, orig_name, offset);
+  sprintf(newname + offset, "%s%s", archname, tmp + 5);
+  return newname;
+}
+
+// return 0 for no file mapped and 1 for file mapped
+int open_file(const char *name, int no_error_non_exist)
+{
+  char *newname = make_new_name(name);
+  if(newname)
+  {
+    int rc = open_file(newname, no_error_non_exist);
+    free(newname);
+    return rc;
+  }
+
   int fd = open(name, O_RDONLY);
   if(fd == -1 && no_error_non_exist && errno == ENOENT)
     return 0;
@@ -159,8 +189,16 @@ int open_file(const char * const name, int no_error_non_exist)
   return open_map(fd, &sb, name);
 }
 
-void map_file_dependencies(const char * const name, int no_error_non_exist)
+void map_file_dependencies(const char *name, int no_error_non_exist)
 {
+  char *newname = make_new_name(name);
+  if(newname)
+  {
+    map_file_dependencies(newname, no_error_non_exist);
+    free(newname);
+    return;
+  }
+
   if(!uid || !gid)
     return;
   int pipe_fd[2];
@@ -221,8 +259,16 @@ void map_file_dependencies(const char * const name, int no_error_non_exist)
   wait(&rc);
 }
 
-void parse_config_file(const char * const config_name, int recurse_count)
+void parse_config_file(const char *config_name, int recurse_count)
 {
+  char *newname = make_new_name(config_name);
+  if(newname)
+  {
+    parse_config_file(newname, recurse_count);
+    free(newname);
+    return;
+  }
+
   struct stat sbuf;
   if(stat(config_name, &sbuf))
   {
@@ -239,20 +285,22 @@ void parse_config_file(const char * const config_name, int recurse_count)
       exit(1);
     }
     int rc;
-    struct dirent entry, *res;
-    while((rc = readdir_r(dirp, &entry, &res)) == 0 && res)
+    struct dirent *entry;
+    errno = 0;
+    while(NULL != (entry = readdir(dirp)))
     {
-      int len = strlen(entry.d_name);
-      if(len > 4 && !strcmp(".cfg", &entry.d_name[len - 4]))
+      int len = strlen(entry->d_name);
+      if(len > 4 && !strcmp(".cfg", &entry->d_name[len - 4]))
       {
         char buf[1024];
-        snprintf(buf, sizeof(buf), "%s/%s", config_name, entry.d_name);
+        snprintf(buf, sizeof(buf), "%s/%s", config_name, entry->d_name);
         parse_config_file(buf, recurse_count);
       }
+      errno = 0;
     }
-    if(rc)
+    if(0 != (rc = errno))
     {
-      log(LOG_ERR, "readdir_r() error for \"%s\"", config_name);
+      log(LOG_ERR, "readdir() error \"%s\" for \"%s\"", strerror(rc), config_name);
       exit(1);
     }
     closedir(dirp);
@@ -399,6 +447,15 @@ int main(int argc, char **argv)
     log(LOG_ERR, "Can't lock memory, exiting");
     exit(1);
   }
+
+  struct utsname un;
+  if(uname(&un))
+  {
+    log(LOG_ERR, "Can't get architecture from uname(2), exiting");
+    exit(1);
+  }
+  archname = un.machine;
+
   parse_config(0);
 
   struct sigaction sa;
